"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SshParser = void 0;
const pegjs_1 = require("pegjs");
const fs_1 = require("@nofrills/fs");
const GrammarError_1 = require("./errors/GrammarError");
class SshParser {
    constructor(cwd, grammar) {
        this.grammar = grammar;
        this.resolver = fs_1.FileResolver.from(cwd, [fs_1.RecursiveStrategy]);
    }
    static from(cwd, grammar = 'config.pegjs') {
        return new SshParser(cwd, grammar);
    }
    generate() {
        return __awaiter(this, void 0, void 0, function* () {
            const grammars = yield this.resolver.find(this.grammar);
            if (grammars.length) {
                const options = {
                    cache: true,
                    optimize: 'speed',
                };
                const grammar = yield fs_1.fs.readFile(grammars[0]);
                const parser = pegjs_1.generate(grammar.toString(), options);
                return parser;
            }
            throw new GrammarError_1.GrammarError(`could not find grammar: ${this.resolver.cwd} ${this.grammar}`);
        });
    }
    parse(input) {
        return __awaiter(this, void 0, void 0, function* () {
            const parser = yield this.generate();
            const config = parser.parse(input);
            return config
                .filter((token) => {
                switch (token.type) {
                    case 'host':
                    case 'identifier':
                    case 'match':
                        return true;
                    default:
                        return false;
                }
            })
                .reduce((previous, token) => {
                if (token.type === 'host' || token.type === 'match') {
                    return [...previous, token];
                }
                return previous;
            }, []);
        });
    }
}
exports.SshParser = SshParser;
//# sourceMappingURL=SshParser.js.map